<?php

namespace Drupal\migrate\Plugin\migrate\process;

use Drupal\Component\Datetime\DateTimePlus;
use Drupal\migrate\MigrateException;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\ProcessPluginBase;
use Drupal\migrate\Row;

/**
 * This plugin converts date/datetime strings from one format into another by
 * applying date storage formats from \DateTime::createFromFormat.
 *
 * Available configuration keys
 *   - from_format: the source format string as accepted by
 *       @link http://php.net/manual/datetime.createfromformat.php
 *       \DateTime::createFromFormat. @endlink
 *   - to_format: the format string as accepted by date() to apply on source.
 *   - timezone: string identifying the required time zone.
 *       @see \Drupal\Component\Datetime\DateTimePlus::__construct()
 *   - settings: keyed array of settings.
 *       @see \Drupal\Component\Datetime\DateTimePlus::__construct()
 *
 * Example usage for date only fields (DATETIME_DATE_STORAGE_FORMAT):
 * The source value is a date string in a particular format:
 * @code
 * source: 01/05/1955
 * @endcode
 * The destination value is the date string in a different format:
 * @code
 * destination: 1955-01-05
 * @endcode
 * The format_date process plugin (containing the formats of the source date and
 * the destination date) would be used like this:
 * @code
 * process:
 *   field_date:
 *     plugin: format_date
 *     from_format: 'm/d/Y'
 *     to_format: 'Y-m-d'
 *     source: event_date
 * @endcode
 *
 * Example usage for datetime fields (DATETIME_DATETIME_STORAGE_FORMAT):
 * The source value is a datetime string in a particular format:
 * @code
 * source: 01/05/1955 10:43:22
 * @endcode
 * The destination value is the datetime string in a different format:
 * @code
 * destination: 1955-01-05T10:43:22
 * @endcode
 * The format_date process plugin (containing the formats of the source datetime
 * and the destination datetime) would be used like this:
 * @code
 * process:
 *   field_time:
 *     plugin: format_date
 *     from_format: 'm/d/Y H:i:s'
 *     to_format: 'Y-m-d\TH:i:s'
 *     source: event_time
 * @endcode
 *
 * Example usage for datetime fields with a timezone and settings:
 * The source value is a datetime string in a particular format:
 * @code
 * source: 2004-12-19T10:19:42-0600
 * @endcode
 * The destination value is the datetime string in a different format:
 * @code
 * destination: 2004-12-19T10:19:42
 * @endcode
 * The format_date process plugin (containing the timezone and the formats of
 * the source datetime and the destination datetime) would be used like this:
 * @code
 * process:
 *   field_time:
 *     plugin: format_date
 *     from_format: 'Y-m-d\TH:i:sO'
 *     to_format: 'Y-m-d\TH:i:s'
 *     timezone: 'America/Managua'
 *     settings:
 *       validate_format: false
 *     source: event_time
 * @endcode
 *
 * @MigrateProcessPlugin(
 *   id = "format_date"
 * )
 */
class FormatDate extends ProcessPluginBase {

  /**
   * {@inheritdoc}
   */
  public function transform($value, MigrateExecutableInterface $migrate_executable, Row $row, $destination_property) {
    if (empty($value)) {
      return '';
    }

    // Validate the configuration.
    if (empty($this->configuration['from_format'])) {
      throw new MigrateException('Format date plugin is missing from_format configuration.');
    }
    if (empty($this->configuration['to_format'])) {
      throw new MigrateException('Format date plugin is missing to_format configuration.');
    }

    $fromFormat = $this->configuration['from_format'];
    $toFormat = $this->configuration['to_format'];
    $timezone = isset($this->configuration['timezone']) ? $this->configuration['timezone'] : NULL;
    $settings = isset($this->configuration['settings']) ? $this->configuration['settings'] : [];

    // Attempts to transform the supplied date using the defined input format.
    // DateTimePlus::createFromFormat can throw exceptions, so we need to
    // explicitly check for problems.
    try {
      $transformed = DateTimePlus::createFromFormat($fromFormat, $value, $timezone, $settings)->format($toFormat);
    }
    catch (\InvalidArgumentException $e) {
      throw new MigrateException(sprintf('Format date plugin could not transform "%s" using the format "%s". Error: %s', $value, $fromFormat, $e->getMessage()), $e->getCode(), $e);
    }
    catch (\UnexpectedValueException $e) {
      throw new MigrateException(sprintf('Format date plugin could not transform "%s" using the format "%s". Error: %s', $value, $fromFormat, $e->getMessage()), $e->getCode(), $e);
    }

    return $transformed;
  }

}
